@@ -1,21 +1,75 @@
 #include "smooth_style.h"
 #include "smooth_rc_style.h"
+#include <stdio.h>
 
-#if GTK1
-  #define GtkSettings  int
-#endif
+smooth_part_style 
+*smooth_tab_part(GtkStyle * style, gboolean for_active_tab)
+{
+  smooth_tab_style tabs = THEME_DATA(style)->tabs;
+  
+  if ((for_active_tab) && (tabs.use_active_tab))
+    return &THEME_DATA(style)->tabs.active_tab;
+  else
+    return THEME_PART(&THEME_DATA(style)->tabs);
+}
+
+gint
+smooth_tab_get_style(GtkStyle * style, gboolean for_active_tab)
+{
+  smooth_tab_style tabs = THEME_DATA(style)->tabs;
+  smooth_part_style active_tab = THEME_DATA(style)->tabs.active_tab;
+  
+  if ((for_active_tab) && (tabs.use_active_tab))
+    return active_tab.style;
+  else
+    return THEME_PART(&tabs)->style;
+}
+
+smooth_fill_style *
+smooth_tab_fill(GtkStyle * style, gboolean for_active_tab)
+{
+  smooth_tab_style tabs = THEME_DATA(style)->tabs;
+  smooth_part_style active_tab = THEME_DATA(style)->tabs.active_tab;
+  
+  if ((for_active_tab) && (tabs.use_active_tab) && (active_tab.use_fill))
+    return &THEME_DATA(style)->tabs.active_tab.fill;
+  else
+    return &THEME_PART(&THEME_DATA(style)->tabs)->fill;
+}
+
+gint
+smooth_tab_edge_line_style(GtkStyle * style, gboolean for_active_tab)
+{
+  smooth_tab_style tabs = THEME_DATA(style)->tabs;
+  smooth_part_style active_tab = THEME_DATA(style)->tabs.active_tab;
+  
+  if ((for_active_tab) && (tabs.use_active_tab) && (active_tab.edge.use_line || active_tab.use_line))
+    return EDGE_LINE_STYLE(style, &active_tab);
+  else
+    return EDGE_LINE_STYLE(style, THEME_PART(&tabs));
+}
+
+gint
+smooth_tab_edge_line_thickness(GtkStyle * style, gboolean for_active_tab)
+{
+  smooth_tab_style tabs = THEME_DATA(style)->tabs;
+  smooth_part_style active_tab = THEME_DATA(style)->tabs.active_tab;
+  
+  if ((for_active_tab) && (tabs.use_active_tab) && (active_tab.edge.use_line || active_tab.use_line))
+    return EDGE_LINE_THICKNESS(style, &active_tab);
+  else
+    return EDGE_LINE_THICKNESS(style, THEME_PART(&tabs));
+}
 
-#if GTK2
 static void   smooth_rc_style_class_init (SmoothRcStyleClass *klass);
 static guint  smooth_rc_style_parse (GtkRcStyle *rc_style, GtkSettings *settings,
 					     GScanner *scanner);
 
 static void   smooth_rc_style_merge (GtkRcStyle *dest, GtkRcStyle *src);
 
-static GtkRcStyleClass *parent_class;
+static GtkRcStyleClass *parent_rc_class;
 
 GType smooth_type_rc_style = 0;
-#endif
 
 enum
   {
@@ -46,6 +100,14 @@ enum
     TOKEN_SOLID,
     TOKEN_ETCHED,
 
+    TOKEN_FOCUS,
+    TOKEN_PATTERN,
+
+    TOKEN_FOREGROUND,
+
+    TOKEN_TABS,
+    TOKEN_ACTIVE_TAB,
+
     TOKEN_PROGRESS,
     
     TOKEN_TROUGH,
@@ -64,12 +126,7 @@ enum
     TOKEN_XPADDING,
     TOKEN_YPADDING,
 
-    TOKEN_TABSTYLE,
-     
-    #if GTK1
-    TOKEN_XTHICKNESS,
-    TOKEN_YTHICKNESS,
-    #endif
+    TOKEN_DEPRECATED_TABSTYLE,//backward compatibility for 0.5.2 & 0.5.4
   };
   
 static struct
@@ -100,8 +157,6 @@ theme_symbols[] =
 
   { "edge",                TOKEN_EDGE },
 
-  { "thickness",           TOKEN_THICKNESS },
-
   { "line",                TOKEN_LINE },
   { "thickness",           TOKEN_THICKNESS },
 
@@ -109,6 +164,14 @@ theme_symbols[] =
   { "solid",               TOKEN_SOLID },
   { "etched",              TOKEN_ETCHED },
 
+  { "focus",               TOKEN_FOCUS },
+  { "pattern",             TOKEN_PATTERN },
+
+  { "foreground",             TOKEN_FOREGROUND },
+
+  { "tabs",                TOKEN_TABS },
+  { "active_tab",          TOKEN_ACTIVE_TAB },
+
   { "progress",            TOKEN_PROGRESS },
 
   { "trough",              TOKEN_TROUGH },
@@ -127,12 +190,7 @@ theme_symbols[] =
   { "xpadding",            TOKEN_XPADDING },
   { "ypadding",            TOKEN_YPADDING },
 
-  { "tab_style",     	   TOKEN_TABSTYLE },
-  
-  #if GTK1
-  { "xthickness",          TOKEN_XTHICKNESS },
-  { "ythickness",          TOKEN_YTHICKNESS },
-  #endif
+  { "tab_style",           TOKEN_DEPRECATED_TABSTYLE },//backward compatibility for 0.5.2 & 0.5.4
 };
 
 static guint n_theme_symbols = sizeof(theme_symbols) / sizeof(theme_symbols[0]);
@@ -142,13 +200,13 @@ TranslateFillStyleName (gchar * str, gint *retval)
 {
 #define is_enum(XX)  (g_ascii_strncasecmp(str, XX, strlen(XX))==0)
   if (is_enum("flat") || is_enum("solid"))
-    *retval = SMOOTH_SOLID_FILL;
+    *retval = SMOOTH_FILL_SOLID;
   else if (is_enum("gradient"))
-    *retval = SMOOTH_GRADIENT_FILL;
+    *retval = SMOOTH_FILL_GRADIENT;
   else if (is_enum("shade_gradient") || is_enum("shaded") || is_enum("shade"))
-    *retval = SMOOTH_SHADE_GRADIENT_FILL;
+    *retval = SMOOTH_FILL_SHADE_GRADIENT;
   else if (is_enum("pixbuf") || is_enum("pixmap") || is_enum("xpm"))
-    *retval = SMOOTH_PIXBUF_FILL;
+    *retval = SMOOTH_FILL_PIXBUF;
   else
     return FALSE; 
   return TRUE;
@@ -226,13 +284,13 @@ TranslateTabStyleName (gchar * str, gint *retval)
 {
 #define is_enum(XX)  (g_ascii_strncasecmp(str, XX, strlen(XX))==0)
   if (is_enum("square") || is_enum("default") || is_enum("normal") || is_enum("standard"))
-    *retval = SMOOTH_NORMAL_TABS;
+    *retval = SMOOTH_TAB_NORMAL;
   else if (is_enum("round"))
-    *retval = SMOOTH_ROUND_TABS;
+    *retval = SMOOTH_TAB_ROUND;
   else if (is_enum("triangle"))
-    *retval = SMOOTH_TRIANGLE_TABS;
+    *retval = SMOOTH_TAB_TRIANGLE;
   else if (is_enum("xpm"))
-    *retval = SMOOTH_XPM_TABS;
+    *retval = SMOOTH_TAB_PIXBUF;
   else
     return FALSE; 
 
@@ -282,6 +340,7 @@ static void part_init (SmoothRcStyle *style, smooth_part_style *part, gint parts
     part->fill.use_color1[i] = FALSE;
     part->fill.use_color2[i] = FALSE;
     part->fill.file_name[i] = NULL;
+    
   }
 
   part->edge.use_line        = FALSE;
@@ -311,10 +370,21 @@ void smooth_rc_style_init (SmoothRcStyle *style)
   style->fill.shade1 = 1.3;
   style->fill.shade2 = 0.7;
   
+  style->buffered_fill = TRUE;
+
   for (i=0; i < 5; i++) {
     style->fill.use_color1[i] = FALSE;
     style->fill.use_color2[i] = FALSE;
     style->fill.file_name[i] = NULL;
+
+    style->focus.use_foreground[i] = FALSE;
+    style->focus.pattern[i] = NULL;
+
+    style->colors.has_light[i] = FALSE;
+    style->colors.has_dark[i] = FALSE;
+    style->colors.has_mid[i] = FALSE;
+    style->colors.has_middark[i] = FALSE;
+    style->colors.has_midlight[i] = FALSE;
   }
 
   style->arrow.style     = DEFAULT_ARROWSTYLE;
@@ -339,13 +409,9 @@ void smooth_rc_style_init (SmoothRcStyle *style)
   part_init(style, THEME_PART(&style->trough), 0);
   style->trough.show_value = DEFAULT_TROUGH_SHOW_VALUE;
   
-  style->tab_style 	 = DEFAULT_TABSTYLE;
-  #if GTK1
-  style->xthickness       = -1;
-  style->ythickness       = -1;
-
-  style->paned_handle_size	  = 10;
-  #endif
+  part_init(style, THEME_PART(&style->tabs), DEFAULT_TABSTYLE);
+  part_init(style, &style->tabs.active_tab, DEFAULT_TABSTYLE);
+  style->tabs.use_active_tab	= FALSE;
 }
 
 static guint 
@@ -625,12 +691,7 @@ static guint theme_parse_fill (GtkSettings  *settings, GScanner *scanner, GToken
 
         token = g_scanner_get_next_token (scanner);
         if (token == G_TOKEN_STRING)
-#ifdef GTK1
-          retval->file_name[state] = gtk_rc_find_pixmap_in_path(scanner, scanner->value.v_string);
-#endif
-#ifdef GTK2
           retval->file_name[state] = gtk_rc_find_pixmap_in_path(settings, scanner, scanner->value.v_string);
-#endif
       }	
       break;
     case TOKEN_QUADRATIC_GRADIENT:
@@ -651,6 +712,80 @@ static guint theme_parse_fill (GtkSettings  *settings, GScanner *scanner, GToken
   return token;
 }
 
+static 
+guint theme_parse_focus (GtkSettings  *settings, GScanner *scanner, GTokenType wanted_token, smooth_focus_style *retval)
+{
+  guint token;
+
+  token = g_scanner_get_next_token (scanner);
+  if (token != wanted_token)
+    {
+      return wanted_token;
+    }
+
+  token = g_scanner_get_next_token (scanner);
+  if (token != G_TOKEN_LEFT_CURLY)
+    return G_TOKEN_LEFT_CURLY;
+
+  token = g_scanner_peek_next_token (scanner);
+  while (token != G_TOKEN_RIGHT_CURLY) {
+    switch (token) {
+    case TOKEN_PATTERN:
+      {
+        GtkStateType state;
+        
+        token = g_scanner_get_next_token (scanner);
+  
+        token = gtk_rc_parse_state (scanner, &state);
+        if (token != G_TOKEN_NONE)
+          return token;
+
+        token = g_scanner_get_next_token (scanner);
+        if (token != G_TOKEN_EQUAL_SIGN)
+          return G_TOKEN_EQUAL_SIGN;
+
+        token = g_scanner_get_next_token (scanner);
+        if (token == G_TOKEN_STRING)
+          if (scanner->value.v_string)
+            retval->pattern[state]= g_strdup(scanner->value.v_string);   
+      }
+
+      break;
+    case TOKEN_FOREGROUND:
+      {
+        GtkStateType state;
+  
+        token = g_scanner_get_next_token (scanner);
+        if (token != TOKEN_FOREGROUND)
+          return TOKEN_FOREGROUND;
+	  
+        token = gtk_rc_parse_state (scanner, &state);
+        if (token != G_TOKEN_NONE)
+          return token;
+  
+        token = g_scanner_get_next_token (scanner);
+        if (token != G_TOKEN_EQUAL_SIGN)
+          return G_TOKEN_EQUAL_SIGN;
+
+        retval->use_foreground[state] = TRUE;
+        token = gtk_rc_parse_color (scanner, &retval->foreground[state]);
+      }
+      break;
+    default:
+      g_scanner_get_next_token (scanner);
+      token = G_TOKEN_RIGHT_CURLY;
+      break;
+    }
+
+    token = g_scanner_peek_next_token (scanner);
+  }
+
+  g_scanner_get_next_token (scanner);
+  token = G_TOKEN_NONE;
+
+  return token;
+}
+
 static guint theme_parse_edge (GtkSettings  *settings, GScanner *scanner, GTokenType wanted_token, smooth_edge_style *retval)
 {
   smooth_edge_style junk_edge;
@@ -747,6 +882,124 @@ static guint theme_parse_arrow (GtkSettings  *settings, GScanner *scanner, GToke
   return token;
 }
 
+static guint theme_parse_active_tab (GtkSettings  *settings, GScanner *scanner, GTokenType wanted_token, smooth_part_style *retval)
+{
+  guint token;
+
+  token = g_scanner_get_next_token (scanner);
+  if (token != wanted_token)
+    {
+      return wanted_token;
+    }
+
+  token = g_scanner_get_next_token (scanner);
+  if (token != G_TOKEN_LEFT_CURLY)
+    return G_TOKEN_LEFT_CURLY;
+
+  token = g_scanner_peek_next_token (scanner);
+  while (token != G_TOKEN_RIGHT_CURLY) {
+    switch (token) {
+      case TOKEN_STYLE:
+	token = theme_parse_custom_enum(scanner, TOKEN_STYLE, TranslateTabStyleName, DEFAULT_TABSTYLE, &THEME_PART(retval)->style);
+	break;
+	
+      case TOKEN_LINE:
+        token = theme_parse_line (settings, scanner, TOKEN_LINE, &THEME_PART(retval)->line);
+        THEME_PART(retval)->use_line = TRUE;
+        break;
+
+      case TOKEN_FILL :
+        token = theme_parse_fill (settings, scanner, TOKEN_FILL, &THEME_PART(retval)->fill);
+        THEME_PART(retval)->use_fill = TRUE;
+        break;
+  
+      case TOKEN_EDGE:
+        token = theme_parse_edge (settings, scanner, TOKEN_EDGE, &THEME_PART(retval)->edge);
+        break;
+
+      case TOKEN_XPADDING:
+        token = theme_parse_int (scanner, TOKEN_XPADDING, 0, &THEME_PART(retval)->xpadding, -25, 25);
+        break;
+
+      case TOKEN_YPADDING:
+        token = theme_parse_int (scanner, TOKEN_YPADDING, 0, &THEME_PART(retval)->ypadding, -25, 25);
+        break;
+    default:
+      g_scanner_get_next_token (scanner);
+      token = G_TOKEN_RIGHT_CURLY;
+      break;
+    }
+
+    token = g_scanner_peek_next_token (scanner);
+  }
+
+  g_scanner_get_next_token (scanner);
+  token = G_TOKEN_NONE;
+
+  return token;
+}
+
+static guint theme_parse_tab (GtkSettings  *settings, GScanner *scanner, GTokenType wanted_token, smooth_tab_style *retval)
+{
+  guint token;
+
+  token = g_scanner_get_next_token (scanner);
+  if (token != wanted_token)
+    {
+      return wanted_token;
+    }
+
+  token = g_scanner_get_next_token (scanner);
+  if (token != G_TOKEN_LEFT_CURLY)
+    return G_TOKEN_LEFT_CURLY;
+
+  token = g_scanner_peek_next_token (scanner);
+  while (token != G_TOKEN_RIGHT_CURLY) {
+    switch (token) {
+      case TOKEN_STYLE:
+	token = theme_parse_custom_enum(scanner, TOKEN_STYLE, TranslateTabStyleName, DEFAULT_TABSTYLE, &THEME_PART(retval)->style);
+	break;
+	
+      case TOKEN_ACTIVE_TAB:
+	token = theme_parse_active_tab (settings, scanner, TOKEN_ACTIVE_TAB, &retval->active_tab);
+	retval->use_active_tab = TRUE;
+	break;	  
+
+      case TOKEN_LINE:
+        token = theme_parse_line (settings, scanner, TOKEN_LINE, &THEME_PART(retval)->line);
+        THEME_PART(retval)->use_line = TRUE;
+        break;
+
+      case TOKEN_FILL :
+        token = theme_parse_fill (settings, scanner, TOKEN_FILL, &THEME_PART(retval)->fill);
+        THEME_PART(retval)->use_fill = TRUE;
+        break;
+  
+      case TOKEN_EDGE:
+        token = theme_parse_edge (settings, scanner, TOKEN_EDGE, &THEME_PART(retval)->edge);
+        break;
+
+      case TOKEN_XPADDING:
+        token = theme_parse_int (scanner, TOKEN_XPADDING, 0, &THEME_PART(retval)->xpadding, -25, 25);
+        break;
+
+      case TOKEN_YPADDING:
+        token = theme_parse_int (scanner, TOKEN_YPADDING, 0, &THEME_PART(retval)->ypadding, -25, 25);
+        break;
+    default:
+      g_scanner_get_next_token (scanner);
+      token = G_TOKEN_RIGHT_CURLY;
+      break;
+    }
+
+    token = g_scanner_peek_next_token (scanner);
+  }
+
+  g_scanner_get_next_token (scanner);
+  token = G_TOKEN_NONE;
+
+  return token;
+}
 
 static guint theme_parse_option (GtkSettings  *settings, GScanner *scanner, GTokenType wanted_token, smooth_option_style *retval)
 {
@@ -1017,164 +1270,10 @@ static guint theme_parse_trough_part (GtkSettings  *settings, GScanner *scanner,
   return token;
 }
 
-#if GTK1
-static void
-smooth_gtk1_style_overides(SmoothRcStyle *smooth_style, SmoothRcProperty * smooth_prop)
-{		       
-  #define is_class(xx) ((xx) && (!strcmp(xx, smooth_prop->class_name)))
-  #define is_prop(xx) ((xx) && (!strcmp(xx, smooth_prop->property_name)))
-
-  if (is_class("GtkRange"))
-  {
-    GtkRangeClass *rangeclass = (GtkRangeClass *)gtk_type_class(gtk_range_get_type());
-    if (is_prop("slider-width")) 
-        rangeclass->slider_width = smooth_prop->value.d.long_data;
-    if (is_prop("slider_width")) 
-        rangeclass->slider_width = smooth_prop->value.d.long_data;
-    if (is_prop("stepper_size")) 
-        rangeclass->stepper_size = smooth_prop->value.d.long_data;
-    if (is_prop("stepper_spacing")) 
-        rangeclass->stepper_slider_spacing = smooth_prop->value.d.long_data;
-  } 
-  else
-    if (is_class("GtkPaned"))
-    {
-      if (is_prop("handle-size") || is_prop("handle_size")) 
-        smooth_style->paned_handle_size = smooth_prop->value.d.long_data;
-    }
-  else
-    if (is_class("GtkScrollbar"))
-    {
-      GtkRangeClass *rangeclass = (GtkRangeClass *)gtk_type_class(gtk_scrollbar_get_type());
-
-      if (is_prop("min_slider_length") || is_prop("min-slider-length")) 
-        rangeclass->min_slider_size = smooth_prop->value.d.long_data;
-    }
-  else
-    if (is_class("GtkScale"))
-    {
-      GtkScaleClass *scaleclass = gtk_type_class(gtk_scale_get_type());
-      GtkRangeClass *rangeclass = (GtkRangeClass *)scaleclass;
-      if (is_prop("slider_width") || is_prop("slider-width")) 
-        rangeclass->slider_width = smooth_prop->value.d.long_data;
-      if (is_prop("slider_length") || is_prop("slider-length")) 
-        scaleclass->slider_length = smooth_prop->value.d.long_data;
-    }
-  else
-    if (is_class("GtkCheckButton"))
-    {
-      GtkCheckButtonClass *checkbuttonclass = (GtkCheckButtonClass*)gtk_type_class (gtk_check_button_get_type ());
-  
-      if (is_prop("indicator_size") || is_prop("indicator-size")) 
-        checkbuttonclass->indicator_size = smooth_prop->value.d.long_data;
-    }
-  else
-    if (is_class("GtkRadioButton"))
-    {
-      GtkCheckButtonClass *checkbuttonclass = (GtkCheckButtonClass*)gtk_type_class (gtk_radio_button_get_type ());
-
-      if (is_prop("indicator_size") || is_prop("indicator-size"))  
-        checkbuttonclass->indicator_size = smooth_prop->value.d.long_data;
-    }
-}
-
-static guint
-gtk_rc_parse_assignment (GScanner      *scanner,
-			 SmoothRcProperty *prop)
-{
-  gboolean scan_identifier = scanner->config->scan_identifier;
-  gboolean scan_symbols = scanner->config->scan_symbols;
-  gboolean identifier_2_string = scanner->config->identifier_2_string;
-  gboolean char_2_token = scanner->config->char_2_token;
-  gboolean scan_identifier_NULL = scanner->config->scan_identifier_NULL;
-  gboolean numbers_2_int = scanner->config->numbers_2_int;
-  gboolean negate = FALSE;
-  guint token;
-
-  /* check that this is an assignment */
-  if (g_scanner_get_next_token (scanner) != '=')
-    return '=';
-
-  /* adjust scanner mode */
-  scanner->config->scan_identifier = TRUE;
-  scanner->config->scan_symbols = FALSE;
-  scanner->config->identifier_2_string = FALSE;
-  scanner->config->char_2_token = TRUE;
-  scanner->config->scan_identifier_NULL = FALSE;
-  scanner->config->numbers_2_int = TRUE;
-
-  /* parse optional sign */
-  if (g_scanner_peek_next_token (scanner) == '-')
-    {
-      g_scanner_get_next_token (scanner); /* eat sign */
-      negate = TRUE;
-    }
-
-  /* parse one of LONG, DOUBLE and STRING or, if that fails, create an unparsed compund */
-  token = g_scanner_peek_next_token (scanner);
-  switch (token)
-    {
-    case G_TOKEN_INT:
-      g_scanner_get_next_token (scanner);
-      prop->value.type = GTK_TYPE_LONG;
-      prop->value.d.long_data = negate ? -scanner->value.v_int : scanner->value.v_int;
-      token = G_TOKEN_NONE;
-      break;
-    case G_TOKEN_FLOAT:
-      g_scanner_get_next_token (scanner);
-      prop->value.type = GTK_TYPE_DOUBLE;
-      prop->value.d.double_data = negate ? -scanner->value.v_float : scanner->value.v_float;
-      token = G_TOKEN_NONE;
-      break;
-    case G_TOKEN_STRING:
-      g_scanner_get_next_token (scanner);
-      if (negate)
-	token = G_TOKEN_INT;
-      else
-	{
-          prop->value.type = GTK_TYPE_STRING;
-          prop->value.d.string_data = scanner->value.v_string;
-	  token = G_TOKEN_NONE;
-	}
-      break;
-    case G_TOKEN_LEFT_CURLY :
-      while (token != G_TOKEN_RIGHT_CURLY) {
-        g_scanner_peek_next_token (scanner);   
-        g_scanner_get_next_token (scanner);
-        token = g_scanner_peek_next_token (scanner);
-      } 
-      g_scanner_get_next_token (scanner);
-      token = G_TOKEN_NONE;      
-      break;      
-    default:
-      g_scanner_get_next_token (scanner);
-      token = G_TOKEN_INT;
-      break;
-    }
-
-  /* restore scanner mode */
-  scanner->config->scan_identifier = scan_identifier;
-  scanner->config->scan_symbols = scan_symbols;
-  scanner->config->identifier_2_string = identifier_2_string;
-  scanner->config->char_2_token = char_2_token;
-  scanner->config->scan_identifier_NULL = scan_identifier_NULL;
-  scanner->config->numbers_2_int = numbers_2_int;
-
-  return token;
-}
-#endif
-
-#if GTK2
 static guint 
 smooth_rc_style_parse (GtkRcStyle *rc_style,
                        GtkSettings  *settings,
 		       GScanner   *scanner)
-#endif
-#if GTK1
-guint 
-smooth_rc_style_parse (GScanner * scanner, 
-                       GtkRcStyle * rc_style)
-#endif
 {
   static GQuark scope_id = 0;
   SmoothRcStyle *smooth_style = NEW_THEME_DATA(rc_style);
@@ -1183,17 +1282,10 @@ smooth_rc_style_parse (GScanner * scanner,
   guint token;
   guint i;
 
-  #if GTK1
-    GtkSettings  *settings=NULL; 
-  
-  smooth_style->refcount = 1;
-  smooth_rc_style_init (smooth_style);
-  #endif
-
   /* Set up a new scope in this scanner */
   if (!scope_id)
     {
-      scope_id = g_quark_from_string ("smooth_theme_engine");
+      scope_id = g_quark_from_string ("theme_engine");
     }
 
   /* If we bail out due to errors, we don't reset the scope, so the
@@ -1203,17 +1295,11 @@ smooth_rc_style_parse (GScanner * scanner,
   /* Check if we already added our symbols to this scope */
   if (!g_scanner_lookup_symbol (scanner, theme_symbols[0].name))
     {
-      #ifdef GTK1
-        g_scanner_freeze_symbol_table (scanner);
-      #endif 	
       for (i = 0; i < n_theme_symbols; i++)
 	{
 	  g_scanner_scope_add_symbol (scanner, scope_id, theme_symbols[i].name, 
 				      GINT_TO_POINTER (theme_symbols[i].token));
 	}
-      #ifdef GTK1
-        g_scanner_thaw_symbol_table (scanner);
-      #endif 	
     }
 
   /* Read to go, now parse the top level */
@@ -1228,6 +1314,9 @@ smooth_rc_style_parse (GScanner * scanner,
 	case TOKEN_FILL :
    	  token = theme_parse_fill (settings, scanner, TOKEN_FILL, &smooth_style->fill);
     	  break;
+        case TOKEN_TABS:
+          token = theme_parse_tab (settings, scanner, TOKEN_TABS, &smooth_style->tabs);
+          break;
 	case TOKEN_REAL_SLIDERS:
 	  token = theme_parse_boolean (scanner, TOKEN_REAL_SLIDERS, DEFAULT_REAL_SLIDERS, &smooth_style->real_sliders);
 	  break;
@@ -1243,6 +1332,9 @@ smooth_rc_style_parse (GScanner * scanner,
 	case TOKEN_ARROW:
 	  token = theme_parse_arrow (settings, scanner, TOKEN_ARROW, &smooth_style->arrow);
 	  break;	  
+        case TOKEN_FOCUS:
+          token = theme_parse_focus (settings, scanner, TOKEN_FOCUS, &smooth_style->focus);
+          break;
 	case TOKEN_PROGRESS:
 	  token = theme_parse_generic_part (settings, scanner, TOKEN_PROGRESS, &smooth_style->progress);
 	  break;
@@ -1255,42 +1347,10 @@ smooth_rc_style_parse (GScanner * scanner,
 	case TOKEN_OPTION:
 	  token = theme_parse_option (settings, scanner, TOKEN_OPTION, &smooth_style->option);
 	  break;
-	case TOKEN_TABSTYLE:
-	  token = theme_parse_custom_enum(scanner, TOKEN_TABSTYLE, TranslateTabStyleName, DEFAULT_TABSTYLE, &smooth_style->tab_style);
+	case TOKEN_DEPRECATED_TABSTYLE:
+	  token = theme_parse_custom_enum(scanner, TOKEN_DEPRECATED_TABSTYLE, TranslateTabStyleName, DEFAULT_TABSTYLE, 
+						&THEME_PART(&smooth_style->tabs)->style);
 	  break;
-	#if GTK1
-	case TOKEN_XTHICKNESS:
-          token = theme_parse_int (scanner, TOKEN_XTHICKNESS, 2, &smooth_style->xthickness, 0, 100);
-	  break;	  
-	case TOKEN_YTHICKNESS:
-          token = theme_parse_int (scanner, TOKEN_YTHICKNESS, 2, &smooth_style->ythickness, 0, 100);
-	  break;	  
-	case G_TOKEN_IDENTIFIER:
-	    {
-	      SmoothRcProperty prop = {NULL, NULL, {1, NULL,}};
-	      
-	      g_scanner_get_next_token (scanner); /* eat type name */
-	      
-	      prop.class_name = g_strdup(scanner->value.v_identifier);
-	      
-	      if (g_scanner_get_next_token (scanner) != ':' ||
-		  g_scanner_get_next_token (scanner) != ':')
-		{
-		  token = ':';
-		  break;
-		}
-	      
-	      g_scanner_get_next_token (scanner);
-
-	      prop.property_name = g_strdup(scanner->value.v_identifier);
-	      token = gtk_rc_parse_assignment (scanner, &prop);
-	      if (token == G_TOKEN_NONE)
-		  smooth_gtk1_style_overides(smooth_style, &prop);
-      	      if (prop.class_name) g_free(prop.class_name);
-      	      if (prop.property_name) g_free(prop.property_name);
-	    }
-	  break;
-	#endif  
 	default:
 	  g_scanner_get_next_token (scanner);
 	  token = G_TOKEN_RIGHT_CURLY;
@@ -1315,7 +1375,7 @@ smooth_rc_style_parse (GScanner * scanner,
   return G_TOKEN_NONE;
 }
 	
-void part_merge (smooth_part_style *dest_part, smooth_part_style *src_part)
+ void part_merge (smooth_part_style *dest_part, smooth_part_style *src_part)
 {
   gint i;
   
@@ -1342,8 +1402,12 @@ void part_merge (smooth_part_style *dest_part, smooth_part_style *src_part)
     if (src_part->fill.use_color2[i])
       color_merge(&dest_part->fill.color2[i], &src_part->fill.color2[i]);
 
-    if (src_part->fill.file_name[i])
+    if (src_part->fill.file_name[i]) {
+      if (dest_part->fill.file_name[i])
+        g_free(dest_part->fill.file_name[i]);
+
       dest_part->fill.file_name[i] = g_strdup(src_part->fill.file_name[i]);
+    }
   }
   
   dest_part->edge.use_line  	  = src_part->edge.use_line;
@@ -1354,38 +1418,20 @@ void part_merge (smooth_part_style *dest_part, smooth_part_style *src_part)
   dest_part->ypadding  	     = src_part->ypadding;
 }
 
-#if GTK2
 static void
 smooth_rc_style_merge (GtkRcStyle * dest,
                        GtkRcStyle * src)
-#endif
-#if GTK1
-void
-smooth_rc_style_merge (GtkRcStyle * dest,
-                       GtkRcStyle * src)
-#endif
 {
   gint i;
-  #if GTK2
-  parent_class->merge (dest, src);
-  #endif
+  parent_rc_class->merge (dest, src);
   
-  #if GTK2
   if (SMOOTH_IS_RC_STYLE (src)) 
-  #endif
     {     
       SmoothRcStyle *src_data = SMOOTH_RC_DATA (src);
       SmoothRcStyle *dest_data = SMOOTH_RC_DATA (dest);
 
       if (!dest_data) {
         dest_data = NEW_THEME_DATA(dest);
-        #if GTK1
-        dest_data->xthickness       = src_data->xthickness;
-        dest_data->ythickness       = src_data->ythickness;
-        dest_data->paned_handle_size = src_data->paned_handle_size;
-
-	dest_data->refcount         = 1;
-        #endif
       } 	
       
       dest_data->real_sliders = src_data->real_sliders;
@@ -1412,8 +1458,44 @@ smooth_rc_style_merge (GtkRcStyle * dest,
         if (src_data->fill.use_color2[i])
           color_merge(&dest_data->fill.color2[i], &src_data->fill.color2[i]);
 
-        if (src_data->fill.file_name[i])
+        dest_data->colors.has_dark[i] = src_data->colors.has_dark[i];
+        if (src_data->colors.has_dark[i])
+          color_merge(&dest_data->colors.dark[i], &src_data->colors.dark[i]);
+
+        dest_data->colors.has_light[i] = src_data->colors.has_light[i];
+        if (src_data->colors.has_light[i])
+          color_merge(&dest_data->colors.light[i], &src_data->colors.light[i]);
+
+        dest_data->colors.has_mid[i] = src_data->colors.has_mid[i];
+        if (src_data->colors.has_mid[i])
+          color_merge(&dest_data->colors.mid[i], &src_data->colors.mid[i]);
+
+        dest_data->colors.has_middark[i] = src_data->colors.has_middark[i];
+        if (src_data->colors.has_middark[i])
+          color_merge(&dest_data->colors.middark[i], &src_data->colors.middark[i]);
+
+        dest_data->colors.has_midlight[i] = src_data->colors.has_midlight[i];
+        if (src_data->colors.has_midlight[i])
+          color_merge(&dest_data->colors.midlight[i], &src_data->colors.midlight[i]);
+
+        if (src_data->fill.file_name[i]) {
+          if (dest_data->fill.file_name[i])
+            g_free(dest_data->fill.file_name[i]);
+
           dest_data->fill.file_name[i] = g_strdup(src_data->fill.file_name[i]);
+        }
+
+        if (src_data->focus.pattern[i]) {
+          if (dest_data->focus.pattern[i])
+            g_free(dest_data->focus.pattern[i]);
+
+          dest_data->focus.pattern[i] = g_strdup(src_data->focus.pattern[i]);
+        }
+
+        dest_data->focus.use_foreground[i] = src_data->focus.use_foreground[i];
+        if (src_data->focus.use_foreground[i])
+          color_merge(&dest_data->focus.foreground[i], &src_data->focus.foreground[i]);
+
       }
 
       dest_data->arrow.style = src_data->arrow.style;
@@ -1438,13 +1520,53 @@ smooth_rc_style_merge (GtkRcStyle * dest,
       
       part_merge(&dest_data->progress,&src_data->progress);
 
-      dest_data->tab_style = src_data->tab_style;
+      part_merge(THEME_PART(&dest_data->tabs), THEME_PART(&src_data->tabs));
+      dest_data->tabs.use_active_tab = src_data->tabs.use_active_tab;
+      part_merge(&dest_data->tabs.active_tab,&src_data->tabs.active_tab);
+
+      dest_data->buffered_fill = src_data->buffered_fill;
 
       SET_THEME_DATA(dest, dest_data);
     }
 }
 
-#if GTK2
+void part_finalize (smooth_part_style *part)
+{
+  gint i;
+  
+  for (i=0; i < 5; i++) {
+    if (part->fill.file_name[i])
+      g_free(part->fill.file_name[i]);
+  }
+}
+
+void smooth_rc_style_dispose (GObject *rc_style)
+{
+  gint i;
+  if (SMOOTH_IS_RC_STYLE (rc_style)) 
+    {     
+      SmoothRcStyle *data = SMOOTH_RC_DATA(rc_style);
+
+      if (data) {  
+        for (i=0; i < 5; i++) {
+          if (data->fill.file_name[i])
+            g_free(data->fill.file_name[i]);
+
+          if (data->focus.pattern[i])
+            g_free(data->focus.pattern[i]);
+        }
+        part_finalize(THEME_PART(&data->grip));
+        part_finalize(THEME_PART(&data->check));
+        part_finalize(THEME_PART(&data->option));
+        part_finalize(THEME_PART(&data->trough));
+        part_finalize(&data->progress);
+        part_finalize(THEME_PART(&data->tabs));
+        part_finalize(&data->tabs.active_tab);
+      }
+      G_OBJECT_CLASS(parent_rc_class)->dispose(G_OBJECT(rc_style));
+   }
+}
+
 /* Create an empty style suitable to this RC style
  */
 static GtkStyle *
@@ -1459,7 +1581,9 @@ static void smooth_rc_style_class_init (SmoothRcStyleClass *klass)
 {
   GtkRcStyleClass *rc_style_class = GTK_RC_STYLE_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
+  parent_rc_class = g_type_class_peek_parent (klass);
+
+  G_OBJECT_CLASS(klass)->dispose = smooth_rc_style_dispose;
 
   rc_style_class->parse        = smooth_rc_style_parse;
   rc_style_class->merge        = smooth_rc_style_merge;
@@ -1486,4 +1610,3 @@ void smooth_rc_style_register_type (GTypeModule *module)
 							      "SmoothRcStyle",
 							      &object_info, 0);
 }
-#endif
